@@ -1,9 +1,9 @@
-#!/usr/bin/python3 -u
 #
 # dulwich - Simple command-line interface to Dulwich
 # Copyright (C) 2008-2011 Jelmer Vernooij <jelmer@jelmer.uk>
 # vim: expandtab
 #
+# SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
 # General Public License as public by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
@@ -34,7 +34,7 @@ import os
 import signal
 import sys
 from getopt import getopt
-from typing import Dict, Optional, Type
+from typing import TYPE_CHECKING, ClassVar, Optional
 
 from dulwich import porcelain
 
@@ -45,12 +45,16 @@ from .objectspec import parse_commit
 from .pack import Pack, sha_to_hex
 from .repo import Repo
 
+if TYPE_CHECKING:
+    from .objects import ObjectID
+    from .refs import Ref
 
-def signal_int(signal, frame):
+
+def signal_int(signal, frame) -> None:
     sys.exit(1)
 
 
-def signal_quit(signal, frame):
+def signal_quit(signal, frame) -> None:
     import pdb
 
     pdb.set_trace()
@@ -59,20 +63,20 @@ def signal_quit(signal, frame):
 class Command:
     """A Dulwich subcommand."""
 
-    def run(self, args):
+    def run(self, args) -> Optional[int]:
         """Run the command."""
         raise NotImplementedError(self.run)
 
 
 class cmd_archive(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = argparse.ArgumentParser()
         parser.add_argument(
             "--remote",
             type=str,
             help="Retrieve archive from specified remote repo",
         )
-        parser.add_argument('committish', type=str, nargs='?')
+        parser.add_argument("committish", type=str, nargs="?")
         args = parser.parse_args(args)
         if args.remote:
             client, path = get_transport_and_path(args.remote)
@@ -84,13 +88,12 @@ class cmd_archive(Command):
             )
         else:
             porcelain.archive(
-                ".", args.committish, outstream=sys.stdout.buffer,
-                errstream=sys.stderr
+                ".", args.committish, outstream=sys.stdout.buffer, errstream=sys.stderr
             )
 
 
 class cmd_add(Command):
-    def run(self, argv):
+    def run(self, argv) -> None:
         parser = argparse.ArgumentParser()
         args = parser.parse_args(argv)
 
@@ -98,7 +101,7 @@ class cmd_add(Command):
 
 
 class cmd_rm(Command):
-    def run(self, argv):
+    def run(self, argv) -> None:
         parser = argparse.ArgumentParser()
         args = parser.parse_args(argv)
 
@@ -106,10 +109,11 @@ class cmd_rm(Command):
 
 
 class cmd_fetch_pack(Command):
-    def run(self, argv):
+    def run(self, argv) -> None:
         parser = argparse.ArgumentParser()
-        parser.add_argument('--all', action='store_true')
-        parser.add_argument('location', nargs='?', type=str)
+        parser.add_argument("--all", action="store_true")
+        parser.add_argument("location", nargs="?", type=str)
+        parser.add_argument("refs", nargs="*", type=str)
         args = parser.parse_args(argv)
         client, path = get_transport_and_path(args.location)
         r = Repo(".")
@@ -117,34 +121,49 @@ class cmd_fetch_pack(Command):
             determine_wants = r.object_store.determine_wants_all
         else:
 
-            def determine_wants(x, **kwargs):
-                return [y for y in args if y not in r.object_store]
+            def determine_wants(
+                refs: dict[Ref, ObjectID], depth: Optional[int] = None
+            ) -> list[ObjectID]:
+                return [y.encode("utf-8") for y in args.refs if y not in r.object_store]
 
         client.fetch(path, r, determine_wants)
 
 
 class cmd_fetch(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
-        opts = dict(opts)
+        dict(opts)
         client, path = get_transport_and_path(args.pop(0))
         r = Repo(".")
-        refs = client.fetch(path, r, progress=sys.stdout.write)
+
+        def progress(msg: bytes) -> None:
+            sys.stdout.buffer.write(msg)
+
+        refs = client.fetch(path, r, progress=progress)
         print("Remote refs:")
         for item in refs.items():
             print("{} -> {}".format(*item))
 
 
+class cmd_for_each_ref(Command):
+    def run(self, args) -> None:
+        parser = argparse.ArgumentParser()
+        parser.add_argument("pattern", type=str, nargs="?")
+        args = parser.parse_args(args)
+        for sha, object_type, ref in porcelain.for_each_ref(".", args.pattern):
+            print(f"{sha.decode()} {object_type.decode()}\t{ref.decode()}")
+
+
 class cmd_fsck(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
-        opts = dict(opts)
-        for (obj, msg) in porcelain.fsck("."):
+        dict(opts)
+        for obj, msg in porcelain.fsck("."):
             print(f"{obj}: {msg}")
 
 
 class cmd_log(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         parser.add_option(
             "--reverse",
@@ -170,22 +189,23 @@ class cmd_log(Command):
 
 
 class cmd_diff(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
 
         r = Repo(".")
         if args == []:
-            commit_id = b'HEAD'
+            commit_id = b"HEAD"
         else:
             commit_id = args[0]
         commit = parse_commit(r, commit_id)
         parent_commit = r[commit.parents[0]]
         porcelain.diff_tree(
-            r, parent_commit.tree, commit.tree, outstream=sys.stdout.buffer)
+            r, parent_commit.tree, commit.tree, outstream=sys.stdout.buffer
+        )
 
 
 class cmd_dump_pack(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
 
         if args == []:
@@ -194,14 +214,13 @@ class cmd_dump_pack(Command):
 
         basename, _ = os.path.splitext(args[0])
         x = Pack(basename)
-        print("Object names checksum: %s" % x.name())
-        print("Checksum: %s" % sha_to_hex(x.get_stored_checksum()))
-        if not x.check():
-            print("CHECKSUM DOES NOT MATCH")
-        print("Length: %d" % len(x))
+        print(f"Object names checksum: {x.name()}")
+        print(f"Checksum: {sha_to_hex(x.get_stored_checksum())}")
+        x.check()
+        print(f"Length: {len(x)}")
         for name in x:
             try:
-                print("\t%s" % x[name])
+                print(f"\t{x[name]}")
             except KeyError as k:
                 print(f"\t{name}: Unable to resolve base {k}")
             except ApplyDeltaError as e:
@@ -209,7 +228,7 @@ class cmd_dump_pack(Command):
 
 
 class cmd_dump_index(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
 
         if args == []:
@@ -224,20 +243,20 @@ class cmd_dump_index(Command):
 
 
 class cmd_init(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", ["bare"])
-        opts = dict(opts)
+        kwopts = dict(opts)
 
         if args == []:
             path = os.getcwd()
         else:
             path = args[0]
 
-        porcelain.init(path, bare=("--bare" in opts))
+        porcelain.init(path, bare=("--bare" in kwopts))
 
 
 class cmd_clone(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         parser.add_option(
             "--bare",
@@ -249,9 +268,28 @@ class cmd_clone(Command):
             "--depth", dest="depth", type=int, help="Depth at which to fetch"
         )
         parser.add_option(
-            "-b", "--branch", dest="branch", type=str,
-            help=("Check out branch instead of branch pointed to by remote "
-                  "HEAD"))
+            "-b",
+            "--branch",
+            dest="branch",
+            type=str,
+            help=("Check out branch instead of branch pointed to by remote " "HEAD"),
+        )
+        parser.add_option(
+            "--refspec",
+            dest="refspec",
+            type=str,
+            help="References to fetch",
+            action="append",
+        )
+        parser.add_option(
+            "--filter",
+            dest="filter_spec",
+            type=str,
+            help="git-rev-list-style object filter",
+        )
+        parser.add_option(
+            "--protocol", dest="protocol", type=int, help="Git protocol version to use"
+        )
         options, args = parser.parse_args(args)
 
         if args == []:
@@ -265,36 +303,44 @@ class cmd_clone(Command):
             target = None
 
         try:
-            porcelain.clone(source, target, bare=options.bare, depth=options.depth,
-                            branch=options.branch)
+            porcelain.clone(
+                source,
+                target,
+                bare=options.bare,
+                depth=options.depth,
+                branch=options.branch,
+                refspec=options.refspec,
+                filter_spec=options.filter_spec,
+                protocol_version=options.protocol,
+            )
         except GitProtocolError as e:
-            print("%s" % e)
+            print(f"{e}")
 
 
 class cmd_commit(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", ["message"])
-        opts = dict(opts)
-        porcelain.commit(".", message=opts["--message"])
+        kwopts = dict(opts)
+        porcelain.commit(".", message=kwopts["--message"])
 
 
 class cmd_commit_tree(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", ["message"])
         if args == []:
             print("usage: dulwich commit-tree tree")
             sys.exit(1)
-        opts = dict(opts)
-        porcelain.commit_tree(".", tree=args[0], message=opts["--message"])
+        kwopts = dict(opts)
+        porcelain.commit_tree(".", tree=args[0], message=kwopts["--message"])
 
 
 class cmd_update_server_info(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         porcelain.update_server_info(".")
 
 
 class cmd_symbolic_ref(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", ["ref-name", "force"])
         if not args:
             print("Usage: dulwich symbolic-ref REF_NAME [--force]")
@@ -305,11 +351,11 @@ class cmd_symbolic_ref(Command):
 
 
 class cmd_pack_refs(Command):
-    def run(self, argv):
+    def run(self, argv) -> None:
         parser = argparse.ArgumentParser()
-        parser.add_argument('--all', action='store_true')
+        parser.add_argument("--all", action="store_true")
         # ignored, we never prune
-        parser.add_argument('--no-prune', action='store_true')
+        parser.add_argument("--no-prune", action="store_true")
 
         args = parser.parse_args(argv)
 
@@ -317,15 +363,15 @@ class cmd_pack_refs(Command):
 
 
 class cmd_show(Command):
-    def run(self, argv):
+    def run(self, argv) -> None:
         parser = argparse.ArgumentParser()
-        parser.add_argument('objectish', type=str, nargs='*')
+        parser.add_argument("objectish", type=str, nargs="*")
         args = parser.parse_args(argv)
         porcelain.show(".", args.objectish or None)
 
 
 class cmd_diff_tree(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
         if len(args) < 2:
             print("Usage: dulwich diff-tree OLD-TREE NEW-TREE")
@@ -334,7 +380,7 @@ class cmd_diff_tree(Command):
 
 
 class cmd_rev_list(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
         if len(args) < 1:
             print("Usage: dulwich rev-list COMMITID...")
@@ -343,7 +389,7 @@ class cmd_rev_list(Command):
 
 
 class cmd_tag(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         parser.add_option(
             "-a",
@@ -361,28 +407,28 @@ class cmd_tag(Command):
 
 
 class cmd_repack(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
-        opts = dict(opts)
+        dict(opts)
         porcelain.repack(".")
 
 
 class cmd_reset(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", ["hard", "soft", "mixed"])
-        opts = dict(opts)
+        kwopts = dict(opts)
         mode = ""
-        if "--hard" in opts:
+        if "--hard" in kwopts:
             mode = "hard"
-        elif "--soft" in opts:
+        elif "--soft" in kwopts:
             mode = "soft"
-        elif "--mixed" in opts:
+        elif "--mixed" in kwopts:
             mode = "mixed"
-        porcelain.reset(".", mode=mode, *args)
+        porcelain.reset(".", mode=mode)
 
 
 class cmd_daemon(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         from dulwich import log_utils
 
         from .protocol import TCP_GIT_PORT
@@ -415,7 +461,7 @@ class cmd_daemon(Command):
 
 
 class cmd_web_daemon(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         from dulwich import log_utils
 
         parser = optparse.OptionParser()
@@ -446,14 +492,14 @@ class cmd_web_daemon(Command):
 
 
 class cmd_write_tree(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
-        sys.stdout.write("%s\n" % porcelain.write_tree("."))
+        sys.stdout.write("{}\n".format(porcelain.write_tree(".")))
 
 
 class cmd_receive_pack(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         if len(args) >= 1:
@@ -464,7 +510,7 @@ class cmd_receive_pack(Command):
 
 
 class cmd_upload_pack(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         if len(args) >= 1:
@@ -475,7 +521,7 @@ class cmd_upload_pack(Command):
 
 
 class cmd_status(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         if len(args) >= 1:
@@ -494,17 +540,17 @@ class cmd_status(Command):
         if status.unstaged:
             sys.stdout.write("Changes not staged for commit:\n\n")
             for name in status.unstaged:
-                sys.stdout.write("\t%s\n" % name.decode(sys.getfilesystemencoding()))
+                sys.stdout.write(f"\t{name.decode(sys.getfilesystemencoding())}\n")
             sys.stdout.write("\n")
         if status.untracked:
             sys.stdout.write("Untracked files:\n\n")
             for name in status.untracked:
-                sys.stdout.write("\t%s\n" % name)
+                sys.stdout.write(f"\t{name}\n")
             sys.stdout.write("\n")
 
 
 class cmd_ls_remote(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         opts, args = getopt(args, "", [])
         if len(args) < 1:
             print("Usage: dulwich ls-remote URL")
@@ -515,7 +561,7 @@ class cmd_ls_remote(Command):
 
 
 class cmd_ls_tree(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         parser.add_option(
             "-r",
@@ -539,20 +585,20 @@ class cmd_ls_tree(Command):
 
 
 class cmd_pack_objects(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         deltify = False
         reuse_deltas = True
         opts, args = getopt(args, "", ["stdout", "deltify", "no-reuse-deltas"])
-        opts = dict(opts)
-        if len(args) < 1 and "--stdout" not in opts.keys():
+        kwopts = dict(opts)
+        if len(args) < 1 and "--stdout" not in kwopts.keys():
             print("Usage: dulwich pack-objects basename")
             sys.exit(1)
         object_ids = [line.strip() for line in sys.stdin.readlines()]
-        if "--deltify" in opts.keys():
+        if "--deltify" in kwopts.keys():
             deltify = True
-        if "--no-reuse-deltas" in opts.keys():
+        if "--no-reuse-deltas" in kwopts.keys():
             reuse_deltas = False
-        if "--stdout" in opts.keys():
+        if "--stdout" in kwopts.keys():
             packf = getattr(sys.stdout, "buffer", sys.stdout)
             idxf = None
             close = []
@@ -562,92 +608,96 @@ class cmd_pack_objects(Command):
             idxf = open(basename + ".idx", "wb")
             close = [packf, idxf]
         porcelain.pack_objects(
-            ".",
-            object_ids,
-            packf,
-            idxf,
-            deltify=deltify,
-            reuse_deltas=reuse_deltas)
+            ".", object_ids, packf, idxf, deltify=deltify, reuse_deltas=reuse_deltas
+        )
         for f in close:
             f.close()
 
 
 class cmd_pull(Command):
-    def run(self, args):
-        parser = optparse.OptionParser()
-        options, args = parser.parse_args(args)
-        try:
-            from_location = args[0]
-        except IndexError:
-            from_location = None
-        porcelain.pull(".", from_location)
+    def run(self, args) -> None:
+        parser = argparse.ArgumentParser()
+        parser.add_argument("from_location", type=str)
+        parser.add_argument("refspec", type=str, nargs="*")
+        parser.add_argument("--filter", type=str, nargs=1)
+        parser.add_argument("--protocol", type=int, nargs=1)
+        args = parser.parse_args(args)
+        porcelain.pull(
+            ".",
+            args.from_location or None,
+            args.refspec or None,
+            filter_spec=args.filter,
+            protocol_version=args.protocol_version or None,
+        )
 
 
 class cmd_push(Command):
-
-    def run(self, argv):
+    def run(self, argv) -> Optional[int]:
         parser = argparse.ArgumentParser()
-        parser.add_argument('-f', '--force', action='store_true', help='Force')
-        parser.add_argument('to_location', type=str)
-        parser.add_argument('refspec', type=str, nargs='*')
+        parser.add_argument("-f", "--force", action="store_true", help="Force")
+        parser.add_argument("to_location", type=str)
+        parser.add_argument("refspec", type=str, nargs="*")
         args = parser.parse_args(argv)
         try:
-            porcelain.push('.', args.to_location, args.refspec or None, force=args.force)
+            porcelain.push(
+                ".", args.to_location, args.refspec or None, force=args.force
+            )
         except porcelain.DivergedBranches:
-            sys.stderr.write('Diverged branches; specify --force to override')
+            sys.stderr.write("Diverged branches; specify --force to override")
             return 1
 
+        return None
+
 
 class cmd_remote_add(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         porcelain.remote_add(".", args[0], args[1])
 
 
 class SuperCommand(Command):
-
-    subcommands: Dict[str, Type[Command]] = {}
-    default_command: Optional[Type[Command]] = None
+    subcommands: ClassVar[dict[str, type[Command]]] = {}
+    default_command: ClassVar[Optional[type[Command]]] = None
 
     def run(self, args):
         if not args and not self.default_command:
-            print("Supported subcommands: %s" % ", ".join(self.subcommands.keys()))
+            print(
+                "Supported subcommands: {}".format(", ".join(self.subcommands.keys()))
+            )
             return False
         cmd = args[0]
         try:
             cmd_kls = self.subcommands[cmd]
         except KeyError:
-            print("No such subcommand: %s" % args[0])
+            print(f"No such subcommand: {args[0]}")
             return False
         return cmd_kls().run(args[1:])
 
 
 class cmd_remote(SuperCommand):
-
-    subcommands = {
+    subcommands: ClassVar[dict[str, type[Command]]] = {
         "add": cmd_remote_add,
     }
 
 
 class cmd_submodule_list(Command):
-    def run(self, argv):
+    def run(self, argv) -> None:
         parser = argparse.ArgumentParser()
         parser.parse_args(argv)
         for path, sha in porcelain.submodule_list("."):
-            sys.stdout.write(f' {sha} {path}\n')
+            sys.stdout.write(f" {sha} {path}\n")
 
 
 class cmd_submodule_init(Command):
-    def run(self, argv):
+    def run(self, argv) -> None:
         parser = argparse.ArgumentParser()
         parser.parse_args(argv)
         porcelain.submodule_init(".")
 
 
 class cmd_submodule(SuperCommand):
-
-    subcommands = {
+    subcommands: ClassVar[dict[str, type[Command]]] = {
         "init": cmd_submodule_init,
     }
 
@@ -666,7 +716,7 @@ class cmd_check_ignore(Command):
 
 
 class cmd_check_mailmap(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         for arg in args:
@@ -675,15 +725,15 @@ class cmd_check_mailmap(Command):
 
 
 class cmd_stash_list(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         for i, entry in porcelain.stash_list("."):
-            print("stash@{%d}: %s" % (i, entry.message.rstrip("\n")))
+            print("stash@{{{}}}: {}".format(i, entry.message.rstrip("\n")))
 
 
 class cmd_stash_push(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         porcelain.stash_push(".")
@@ -691,7 +741,7 @@ class cmd_stash_push(Command):
 
 
 class cmd_stash_pop(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         porcelain.stash_pop(".")
@@ -699,8 +749,7 @@ class cmd_stash_pop(Command):
 
 
 class cmd_stash(SuperCommand):
-
-    subcommands = {
+    subcommands: ClassVar[dict[str, type[Command]]] = {
         "list": cmd_stash_list,
         "pop": cmd_stash_pop,
         "push": cmd_stash_push,
@@ -708,7 +757,7 @@ class cmd_stash(SuperCommand):
 
 
 class cmd_ls_files(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         for name in porcelain.ls_files("."):
@@ -716,14 +765,14 @@ class cmd_ls_files(Command):
 
 
 class cmd_describe(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         options, args = parser.parse_args(args)
         print(porcelain.describe("."))
 
 
 class cmd_help(Command):
-    def run(self, args):
+    def run(self, args) -> None:
         parser = optparse.OptionParser()
         parser.add_option(
             "-a",
@@ -737,7 +786,7 @@ class cmd_help(Command):
         if options.all:
             print("Available commands:")
             for cmd in sorted(commands):
-                print("  %s" % cmd)
+                print(f"  {cmd}")
         else:
             print(
                 """\
@@ -765,6 +814,7 @@ commands = {
     "dump-index": cmd_dump_index,
     "fetch-pack": cmd_fetch_pack,
     "fetch": cmd_fetch,
+    "for-each-ref": cmd_for_each_ref,
     "fsck": cmd_fsck,
     "help": cmd_help,
     "init": cmd_init,
@@ -800,20 +850,20 @@ def main(argv=None):
         argv = sys.argv[1:]
 
     if len(argv) < 1:
-        print("Usage: dulwich <%s> [OPTIONS...]" % ("|".join(commands.keys())))
+        print("Usage: dulwich <{}> [OPTIONS...]".format("|".join(commands.keys())))
         return 1
 
     cmd = argv[0]
     try:
         cmd_kls = commands[cmd]
     except KeyError:
-        print("No such subcommand: %s" % cmd)
+        print(f"No such subcommand: {cmd}")
         return 1
     # TODO(jelmer): Return non-0 on errors
     return cmd_kls().run(argv[1:])
 
 
-def _main():
+def _main() -> None:
     if "DULWICH_PDB" in os.environ and getattr(signal, "SIGQUIT", None):
         signal.signal(signal.SIGQUIT, signal_quit)  # type: ignore
     signal.signal(signal.SIGINT, signal_int)
