@@ -1,6 +1,7 @@
 # config.py - Reading and writing Git config files
 # Copyright (C) 2011-2013 Jelmer Vernooij <jelmer@jelmer.uk>
 #
+# SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
 # General Public License as public by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
@@ -392,12 +393,11 @@ def _parse_string(value: bytes) -> bytes:
                 v = _ESCAPE_TABLE[value[i]]
             except IndexError as exc:
                 raise ValueError(
-                    "escape character in %r at %d before end of string" % (value, i)
+                    f"escape character in {value!r} at {i} before end of string"
                 ) from exc
             except KeyError as exc:
                 raise ValueError(
-                    "escape character followed by unknown character "
-                    "%s at %d in %r" % (value[i], i, value)
+                    f"escape character followed by unknown character {value[i]!r} at {i} in {value!r}"
                 ) from exc
             if whitespace:
                 ret.extend(whitespace)
