@@ -2,6 +2,7 @@
 # Copyright (C) 2007 James Westby <jw+debian@jameswestby.net>
 # Copyright (C) 2008-2013 Jelmer Vernooij <jelmer@jelmer.uk>
 #
+# SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
 # General Public License as public by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
@@ -120,7 +121,7 @@ def hex_to_sha(hex):
         raise ValueError(exc.args[0]) from exc
 
 
-def valid_hexsha(hex):
+def valid_hexsha(hex) -> bool:
     if len(hex) != 40:
         return False
     try:
@@ -161,14 +162,14 @@ def object_header(num_type: int, length: int) -> bytes:
     """Return an object header for the given numeric type and text length."""
     cls = object_class(num_type)
     if cls is None:
-        raise AssertionError("unsupported class type num: %d" % num_type)
+        raise AssertionError(f"unsupported class type num: {num_type}")
     return cls.type_name + b" " + str(length).encode("ascii") + b"\0"
 
 
 def serializable_property(name: str, docstring: Optional[str] = None):
     """A property that helps tracking whether serialization is necessary."""
 
-    def set(obj, value):
+    def set(obj, value) -> None:
         setattr(obj, "_" + name, value)
         obj._needs_serialization = True
 
@@ -189,7 +190,7 @@ def object_class(type: Union[bytes, int]) -> Optional[type["ShaFile"]]:
     return _TYPE_MAP.get(type, None)
 
 
-def check_hexsha(hex, error_msg):
+def check_hexsha(hex, error_msg) -> None:
     """Check if a string is a valid hex sha string.
 
     Args:
@@ -202,7 +203,7 @@ def check_hexsha(hex, error_msg):
         raise ObjectFormatException(f"{error_msg} {hex}")
 
 
-def check_identity(identity: bytes, error_msg: str) -> None:
+def check_identity(identity: Optional[bytes], error_msg: str) -> None:
     """Check if the specified identity is valid.
 
     This will raise an exception if the identity is not valid.
@@ -211,6 +212,8 @@ def check_identity(identity: bytes, error_msg: str) -> None:
       identity: Identity string
       error_msg: Error message to use in exception
     """
+    if identity is None:
+        raise ObjectFormatException(error_msg)
     email_start = identity.find(b"<")
     email_end = identity.find(b">")
     if not all(
@@ -226,7 +229,7 @@ def check_identity(identity: bytes, error_msg: str) -> None:
         raise ObjectFormatException(error_msg)
 
 
-def check_time(time_seconds):
+def check_time(time_seconds) -> None:
     """Check if the specified time is not prone to overflow error.
 
     This will raise an exception if the time is not valid.
@@ -270,7 +273,7 @@ class FixedSha:
 class ShaFile:
     """A git SHA file."""
 
-    __slots__ = ("_chunked_text", "_sha", "_needs_serialization")
+    __slots__ = ("_chunked_text", "_needs_serialization", "_sha")
 
     _needs_serialization: bool
     type_name: bytes
@@ -384,7 +387,7 @@ class ShaFile:
         num_type = (ord(magic[0:1]) >> 4) & 7
         obj_class = object_class(num_type)
         if not obj_class:
-            raise ObjectFormatException("Not a known type %d" % num_type)
+            raise ObjectFormatException(f"Not a known type {num_type}")
         return obj_class()
 
     def _parse_object(self, map) -> None:
@@ -461,7 +464,7 @@ class ShaFile:
         """
         cls = object_class(type_num)
         if cls is None:
-            raise AssertionError("unsupported class type num: %d" % type_num)
+            raise AssertionError(f"unsupported class type num: {type_num}")
         obj = cls()
         obj.set_raw_string(string, sha)
         return obj
@@ -479,7 +482,7 @@ class ShaFile:
         """
         cls = object_class(type_num)
         if cls is None:
-            raise AssertionError("unsupported class type num: %d" % type_num)
+            raise AssertionError(f"unsupported class type num: {type_num}")
         obj = cls()
         obj.set_raw_chunks(chunks, sha)
         return obj
@@ -491,7 +494,7 @@ class ShaFile:
         obj.set_raw_string(string)
         return obj
 
-    def _check_has_member(self, member, error_msg):
+    def _check_has_member(self, member, error_msg) -> None:
         """Check that the object has a given member variable.
 
         Args:
@@ -548,7 +551,7 @@ class ShaFile:
         """Create a new copy of this SHA1 object from its raw string."""
         obj_class = object_class(self.type_num)
         if obj_class is None:
-            raise AssertionError("invalid type num %d" % self.type_num)
+            raise AssertionError(f"invalid type num {self.type_num}")
         return obj_class.from_raw_string(self.type_num, self.as_raw_string(), self.id)
 
     @property
@@ -598,7 +601,7 @@ class Blob(ShaFile):
     def _get_data(self):
         return self.as_raw_string()
 
-    def _set_data(self, data):
+    def _set_data(self, data) -> None:
         self.set_raw_string(data)
 
     data = property(
@@ -608,13 +611,13 @@ class Blob(ShaFile):
     def _get_chunked(self):
         return self._chunked_text
 
-    def _set_chunked(self, chunks: list[bytes]):
+    def _set_chunked(self, chunks: list[bytes]) -> None:
         self._chunked_text = chunks
 
     def _serialize(self):
         return self._chunked_text
 
-    def _deserialize(self, chunks):
+    def _deserialize(self, chunks) -> None:
         self._chunked_text = chunks
 
     chunked = property(
@@ -630,7 +633,7 @@ class Blob(ShaFile):
             raise NotBlobError(path)
         return blob
 
-    def check(self):
+    def check(self) -> None:
         """Check this object for internal consistency.
 
         Raises:
@@ -728,8 +731,8 @@ def _format_message(headers, body):
         yield git_line(field, lines[0])
         for line in lines[1:]:
             yield b" " + line + b"\n"
+    yield b"\n"  # There must be a new line after the headers
     if body:
-        yield b"\n"  # There must be a new line after the headers
         yield body
 
 
@@ -740,15 +743,15 @@ class Tag(ShaFile):
     type_num = 4
 
     __slots__ = (
-        "_tag_timezone_neg_utc",
+        "_message",
         "_name",
-        "_object_sha",
         "_object_class",
+        "_object_sha",
+        "_signature",
         "_tag_time",
         "_tag_timezone",
+        "_tag_timezone_neg_utc",
         "_tagger",
-        "_message",
-        "_signature",
     )
 
     _tagger: Optional[bytes]
@@ -759,7 +762,7 @@ class Tag(ShaFile):
         self._tag_time = None
         self._tag_timezone = None
         self._tag_timezone_neg_utc = False
-        self._signature = None
+        self._signature: Optional[bytes] = None
 
     @classmethod
     def from_path(cls, filename):
@@ -768,7 +771,7 @@ class Tag(ShaFile):
             raise NotTagError(filename)
         return tag
 
-    def check(self):
+    def check(self) -> None:
         """Check this object for internal consistency.
 
         Raises:
@@ -829,7 +832,7 @@ class Tag(ShaFile):
             body = (self.message or b"") + (self._signature or b"")
         return list(_format_message(headers, body))
 
-    def _deserialize(self, chunks):
+    def _deserialize(self, chunks) -> None:
         """Grab the metadata attached to the tag."""
         self._tagger = None
         self._tag_time = None
@@ -866,7 +869,9 @@ class Tag(ShaFile):
                         self._message = value[:sig_idx]
                         self._signature = value[sig_idx:]
             else:
-                raise ObjectFormatException(f"Unknown field {field}")
+                raise ObjectFormatException(
+                    f"Unknown field {field.decode('ascii', 'replace')}"
+                )
 
     def _get_object(self):
         """Get the object pointed to by this tag.
@@ -875,7 +880,7 @@ class Tag(ShaFile):
         """
         return (self._object_class, self._object_sha)
 
-    def _set_object(self, value):
+    def _set_object(self, value) -> None:
         (self._object_class, self._object_sha) = value
         self._needs_serialization = True
 
@@ -897,7 +902,7 @@ class Tag(ShaFile):
 
     signature = serializable_property("signature", "Optional detached GPG signature")
 
-    def sign(self, keyid: Optional[str] = None):
+    def sign(self, keyid: Optional[str] = None) -> None:
         import gpg
 
         with gpg.Context(armor=True) as c:
@@ -913,6 +918,16 @@ class Tag(ShaFile):
                     self.as_raw_string(), mode=gpg.constants.sig.mode.DETACH
                 )
 
+    def raw_without_sig(self) -> bytes:
+        """Return raw string serialization without the GPG/SSH signature.
+
+        self.signature is a signature for the returned raw byte string serialization.
+        """
+        ret = self.as_raw_string()
+        if self._signature:
+            ret = ret[: -len(self._signature)]
+        return ret
+
     def verify(self, keyids: Optional[Iterable[str]] = None) -> None:
         """Verify GPG signature for this tag (if it is signed).
 
@@ -934,7 +949,7 @@ class Tag(ShaFile):
 
         with gpg.Context() as ctx:
             data, result = ctx.verify(
-                self.as_raw_string()[: -len(self._signature)],
+                self.raw_without_sig(),
                 signature=self._signature,
             )
             if keyids:
@@ -1118,7 +1133,7 @@ class Tree(ShaFile):
     def __iter__(self):
         return iter(self._entries)
 
-    def add(self, name, mode, hexsha):
+    def add(self, name, mode, hexsha) -> None:
         """Add an entry to the tree.
 
         Args:
@@ -1147,7 +1162,7 @@ class Tree(ShaFile):
         """
         return list(self.iteritems())
 
-    def _deserialize(self, chunks):
+    def _deserialize(self, chunks) -> None:
         """Grab the entries in the tree."""
         try:
             parsed_entries = parse_tree(b"".join(chunks))
@@ -1158,7 +1173,7 @@ class Tree(ShaFile):
         # genexp.
         self._entries = {n: (m, s) for n, m, s in parsed_entries}
 
-    def check(self):
+    def check(self) -> None:
         """Check this object for internal consistency.
 
         Raises:
@@ -1245,7 +1260,7 @@ def parse_timezone(text):
     if sign == b"-":
         offset = -offset
     unnecessary_negative_timezone = offset >= 0 and sign == b"-"
-    signum = (offset < 0) and -1 or 1
+    signum = ((offset < 0) and -1) or 1
     offset = abs(offset)
     hours = int(offset / 100)
     minutes = offset % 100
@@ -1270,7 +1285,7 @@ def format_timezone(offset, unnecessary_negative_timezone=False):
         offset = -offset
     else:
         sign = "+"
-    return ("%c%02d%02d" % (sign, offset / 3600, (offset / 60) % 60)).encode("ascii")
+    return ("%c%02d%02d" % (sign, offset / 3600, (offset / 60) % 60)).encode("ascii")  # noqa: UP031
 
 
 def parse_time_entry(value):
@@ -1365,32 +1380,32 @@ class Commit(ShaFile):
     type_num = 1
 
     __slots__ = (
-        "_parents",
-        "_encoding",
-        "_extra",
-        "_author_timezone_neg_utc",
-        "_commit_timezone_neg_utc",
-        "_commit_time",
+        "_author",
         "_author_time",
         "_author_timezone",
+        "_author_timezone_neg_utc",
+        "_commit_time",
         "_commit_timezone",
-        "_author",
+        "_commit_timezone_neg_utc",
         "_committer",
-        "_tree",
-        "_message",
-        "_mergetag",
+        "_encoding",
+        "_extra",
         "_gpgsig",
+        "_mergetag",
+        "_message",
+        "_parents",
+        "_tree",
     )
 
     def __init__(self) -> None:
         super().__init__()
         self._parents: list[bytes] = []
-        self._encoding = None
+        self._encoding: Optional[bytes] = None
         self._mergetag: list[Tag] = []
-        self._gpgsig = None
-        self._extra: list[tuple[bytes, bytes]] = []
-        self._author_timezone_neg_utc = False
-        self._commit_timezone_neg_utc = False
+        self._gpgsig: Optional[bytes] = None
+        self._extra: list[tuple[bytes, Optional[bytes]]] = []
+        self._author_timezone_neg_utc: Optional[bool] = False
+        self._commit_timezone_neg_utc: Optional[bool] = False
 
     @classmethod
     def from_path(cls, path):
@@ -1399,7 +1414,7 @@ class Commit(ShaFile):
             raise NotCommitError(path)
         return commit
 
-    def _deserialize(self, chunks):
+    def _deserialize(self, chunks) -> None:
         self._parents = []
         self._extra = []
         self._tree = None
@@ -1444,7 +1459,7 @@ class Commit(ShaFile):
             (self._commit_timezone, self._commit_timezone_neg_utc),
         ) = commit_info
 
-    def check(self):
+    def check(self) -> None:
         """Check this object for internal consistency.
 
         Raises:
@@ -1490,7 +1505,7 @@ class Commit(ShaFile):
 
         # TODO: optionally check for duplicate parents
 
-    def sign(self, keyid: Optional[str] = None):
+    def sign(self, keyid: Optional[str] = None) -> None:
         import gpg
 
         with gpg.Context(armor=True) as c:
@@ -1506,7 +1521,18 @@ class Commit(ShaFile):
                     self.as_raw_string(), mode=gpg.constants.sig.mode.DETACH
                 )
 
-    def verify(self, keyids: Optional[Iterable[str]] = None):
+    def raw_without_sig(self) -> bytes:
+        """Return raw string serialization without the GPG/SSH signature.
+
+        self.gpgsig is a signature for the returned raw byte string serialization.
+        """
+        tmp = self.copy()
+        assert isinstance(tmp, Commit)
+        tmp._gpgsig = None
+        tmp.gpgsig = None
+        return tmp.as_raw_string()
+
+    def verify(self, keyids: Optional[Iterable[str]] = None) -> None:
         """Verify GPG signature for this commit (if it is signed).
 
         Args:
@@ -1526,11 +1552,8 @@ class Commit(ShaFile):
         import gpg
 
         with gpg.Context() as ctx:
-            self_without_gpgsig = self.copy()
-            self_without_gpgsig._gpgsig = None
-            self_without_gpgsig.gpgsig = None
             data, result = ctx.verify(
-                self_without_gpgsig.as_raw_string(),
+                self.raw_without_sig(),
                 signature=self._gpgsig,
             )
             if keyids:
@@ -1583,7 +1606,7 @@ class Commit(ShaFile):
         """Return a list of parents of this commit."""
         return self._parents
 
-    def _set_parents(self, value):
+    def _set_parents(self, value) -> None:
         """Set a list of parents of this commit."""
         self._needs_serialization = True
         self._parents = value
