@@ -1,6 +1,7 @@
 # __init__.py -- The tests for dulwich
 # Copyright (C) 2007 James Westby <jw+debian@jameswestby.net>
 #
+# SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
 # General Public License as public by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
@@ -21,11 +22,11 @@
 """Tests for Dulwich."""
 
 __all__ = [
+    "BlackboxTestCase",
     "SkipTest",
     "TestCase",
-    "BlackboxTestCase",
-    "skipIf",
     "expectedFailure",
+    "skipIf",
 ]
 
 import doctest
@@ -43,13 +44,13 @@ from unittest import TestCase as _TestCase
 
 
 class TestCase(_TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         super().setUp()
         self.overrideEnv("HOME", "/nonexistent")
         self.overrideEnv("GIT_CONFIG_NOSYSTEM", "1")
 
-    def overrideEnv(self, name, value):
-        def restore():
+    def overrideEnv(self, name, value) -> None:
+        def restore() -> None:
             if oldval is not None:
                 os.environ[name] = oldval
             else:
@@ -168,7 +169,7 @@ def tutorial_test_suite():
 
     to_restore = []
 
-    def overrideEnv(name, value):
+    def overrideEnv(name, value) -> None:
         oldval = os.environ.get(name)
         if value is not None:
             os.environ[name] = value
@@ -176,7 +177,7 @@ def tutorial_test_suite():
             del os.environ[name]
         to_restore.append((name, oldval))
 
-    def setup(test):
+    def setup(test) -> None:
         test.__old_cwd = os.getcwd()
         test.tempdir = tempfile.mkdtemp()
         test.globs.update({"tempdir": test.tempdir})
@@ -184,7 +185,7 @@ def tutorial_test_suite():
         overrideEnv("HOME", "/nonexistent")
         overrideEnv("GIT_CONFIG_NOSYSTEM", "1")
 
-    def teardown(test):
+    def teardown(test) -> None:
         os.chdir(test.__old_cwd)
         shutil.rmtree(test.tempdir)
         for name, oldval in to_restore:
