@@ -1,6 +1,7 @@
 # test_index.py -- Tests for the git index
 # Copyright (C) 2008-2009 Jelmer Vernooij <jelmer@jelmer.uk>
 #
+# SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
 # General Public License as public by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
@@ -54,7 +55,7 @@ from dulwich.repo import Repo
 from . import TestCase, skipIf
 
 
-def can_symlink():
+def can_symlink() -> bool:
     """Return whether running process can create symlinks."""
     if sys.platform != "win32":
         # Platforms other than Windows should allow symlinks without issues.
@@ -77,19 +78,19 @@ class IndexTestCase(TestCase):
 
 
 class SimpleIndexTestCase(IndexTestCase):
-    def test_len(self):
+    def test_len(self) -> None:
         self.assertEqual(1, len(self.get_simple_index("index")))
 
-    def test_iter(self):
+    def test_iter(self) -> None:
         self.assertEqual([b"bla"], list(self.get_simple_index("index")))
 
-    def test_iterobjects(self):
+    def test_iterobjects(self) -> None:
         self.assertEqual(
             [(b"bla", b"e69de29bb2d1d6434b8b29ae775ad8c2e48c5391", 33188)],
             list(self.get_simple_index("index").iterobjects()),
         )
 
-    def test_getitem(self):
+    def test_getitem(self) -> None:
         self.assertEqual(
             IndexEntry(
                 (1230680220, 0),
@@ -105,12 +106,12 @@ class SimpleIndexTestCase(IndexTestCase):
             self.get_simple_index("index")[b"bla"],
         )
 
-    def test_empty(self):
+    def test_empty(self) -> None:
         i = self.get_simple_index("notanindex")
         self.assertEqual(0, len(i))
         self.assertFalse(os.path.exists(i._filename))
 
-    def test_against_empty_tree(self):
+    def test_against_empty_tree(self) -> None:
         i = self.get_simple_index("index")
         changes = list(i.changes_from_tree(MemoryObjectStore(), None))
         self.assertEqual(1, len(changes))
@@ -120,15 +121,15 @@ class SimpleIndexTestCase(IndexTestCase):
 
 
 class SimpleIndexWriterTestCase(IndexTestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         IndexTestCase.setUp(self)
         self.tempdir = tempfile.mkdtemp()
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         IndexTestCase.tearDown(self)
         shutil.rmtree(self.tempdir)
 
-    def test_simple_write(self):
+    def test_simple_write(self) -> None:
         entries = [
             (
                 SerializedIndexEntry(
@@ -156,15 +157,15 @@ class SimpleIndexWriterTestCase(IndexTestCase):
 
 
 class ReadIndexDictTests(IndexTestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         IndexTestCase.setUp(self)
         self.tempdir = tempfile.mkdtemp()
 
-    def tearDown(self):
+    def tearDown(self) -> None:
         IndexTestCase.tearDown(self)
         shutil.rmtree(self.tempdir)
 
-    def test_simple_write(self):
+    def test_simple_write(self) -> None:
         entries = {
             b"barbla": IndexEntry(
                 (1230680220, 0),
@@ -187,11 +188,11 @@ class ReadIndexDictTests(IndexTestCase):
 
 
 class CommitTreeTests(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         super().setUp()
         self.store = MemoryObjectStore()
 
-    def test_single_blob(self):
+    def test_single_blob(self) -> None:
         blob = Blob()
         blob.data = b"foo"
         self.store.add_object(blob)
@@ -201,7 +202,7 @@ class CommitTreeTests(TestCase):
         self.assertEqual((stat.S_IFREG, blob.id), self.store[rootid][b"bla"])
         self.assertEqual({rootid, blob.id}, set(self.store._data.keys()))
 
-    def test_nested(self):
+    def test_nested(self) -> None:
         blob = Blob()
         blob.data = b"foo"
         self.store.add_object(blob)
@@ -216,49 +217,49 @@ class CommitTreeTests(TestCase):
 
 
 class CleanupModeTests(TestCase):
-    def assertModeEqual(self, expected, got):
+    def assertModeEqual(self, expected, got) -> None:
         self.assertEqual(expected, got, f"{expected:o} != {got:o}")
 
-    def test_file(self):
+    def test_file(self) -> None:
         self.assertModeEqual(0o100644, cleanup_mode(0o100000))
 
-    def test_executable(self):
+    def test_executable(self) -> None:
         self.assertModeEqual(0o100755, cleanup_mode(0o100711))
         self.assertModeEqual(0o100755, cleanup_mode(0o100700))
 
-    def test_symlink(self):
+    def test_symlink(self) -> None:
         self.assertModeEqual(0o120000, cleanup_mode(0o120711))
 
-    def test_dir(self):
+    def test_dir(self) -> None:
         self.assertModeEqual(0o040000, cleanup_mode(0o40531))
 
-    def test_submodule(self):
+    def test_submodule(self) -> None:
         self.assertModeEqual(0o160000, cleanup_mode(0o160744))
 
 
 class WriteCacheTimeTests(TestCase):
-    def test_write_string(self):
+    def test_write_string(self) -> None:
         f = BytesIO()
         self.assertRaises(TypeError, write_cache_time, f, "foo")
 
-    def test_write_int(self):
+    def test_write_int(self) -> None:
         f = BytesIO()
         write_cache_time(f, 434343)
         self.assertEqual(struct.pack(">LL", 434343, 0), f.getvalue())
 
-    def test_write_tuple(self):
+    def test_write_tuple(self) -> None:
         f = BytesIO()
         write_cache_time(f, (434343, 21))
         self.assertEqual(struct.pack(">LL", 434343, 21), f.getvalue())
 
-    def test_write_float(self):
+    def test_write_float(self) -> None:
         f = BytesIO()
         write_cache_time(f, 434343.000000021)
         self.assertEqual(struct.pack(">LL", 434343, 21), f.getvalue())
 
 
 class IndexEntryFromStatTests(TestCase):
-    def test_simple(self):
+    def test_simple(self) -> None:
         st = os.stat_result(
             (
                 16877,
@@ -289,7 +290,7 @@ class IndexEntryFromStatTests(TestCase):
             ),
         )
 
-    def test_override_mode(self):
+    def test_override_mode(self) -> None:
         st = os.stat_result(
             (
                 stat.S_IFREG + 0o644,
@@ -322,19 +323,19 @@ class IndexEntryFromStatTests(TestCase):
 
 
 class BuildIndexTests(TestCase):
-    def assertReasonableIndexEntry(self, index_entry, mode, filesize, sha):
+    def assertReasonableIndexEntry(self, index_entry, mode, filesize, sha) -> None:
         self.assertEqual(index_entry.mode, mode)  # mode
         self.assertEqual(index_entry.size, filesize)  # filesize
         self.assertEqual(index_entry.sha, sha)  # sha
 
-    def assertFileContents(self, path, contents, symlink=False):
+    def assertFileContents(self, path, contents, symlink=False) -> None:
         if symlink:
             self.assertEqual(os.readlink(path), contents)
         else:
             with open(path, "rb") as f:
                 self.assertEqual(f.read(), contents)
 
-    def test_empty(self):
+    def test_empty(self) -> None:
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
         with Repo.init(repo_dir) as repo:
@@ -352,7 +353,7 @@ class BuildIndexTests(TestCase):
             # Verify no files
             self.assertEqual([".git"], os.listdir(repo.path))
 
-    def test_git_dir(self):
+    def test_git_dir(self) -> None:
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
         with Repo.init(repo_dir) as repo:
@@ -386,7 +387,7 @@ class BuildIndexTests(TestCase):
             )
             self.assertFileContents(epath, b"d")
 
-    def test_nonempty(self):
+    def test_nonempty(self) -> None:
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
         with Repo.init(repo_dir) as repo:
@@ -441,7 +442,7 @@ class BuildIndexTests(TestCase):
             self.assertEqual(["d"], sorted(os.listdir(os.path.join(repo.path, "c"))))
 
     @skipIf(not getattr(os, "sync", None), "Requires sync support")
-    def test_norewrite(self):
+    def test_norewrite(self) -> None:
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
         with Repo.init(repo_dir) as repo:
@@ -483,7 +484,7 @@ class BuildIndexTests(TestCase):
                 self.assertEqual(b"file a", fh.read())
 
     @skipIf(not can_symlink(), "Requires symlink support")
-    def test_symlink(self):
+    def test_symlink(self) -> None:
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
         with Repo.init(repo_dir) as repo:
@@ -515,7 +516,7 @@ class BuildIndexTests(TestCase):
             )
             self.assertFileContents(epath, "d", symlink=True)
 
-    def test_no_decode_encode(self):
+    def test_no_decode_encode(self) -> None:
         repo_dir = tempfile.mkdtemp()
         repo_dir_bytes = os.fsencode(repo_dir)
         self.addCleanup(shutil.rmtree, repo_dir)
@@ -560,7 +561,7 @@ class BuildIndexTests(TestCase):
 
             self.assertTrue(os.path.exists(utf8_path))
 
-    def test_git_submodule(self):
+    def test_git_submodule(self) -> None:
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
         with Repo.init(repo_dir) as repo:
@@ -600,7 +601,7 @@ class BuildIndexTests(TestCase):
             self.assertEqual(index[b"c"].mode, S_IFGITLINK)  # mode
             self.assertEqual(index[b"c"].sha, c.id)  # sha
 
-    def test_git_submodule_exists(self):
+    def test_git_submodule_exists(self) -> None:
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
         with Repo.init(repo_dir) as repo:
@@ -643,7 +644,7 @@ class BuildIndexTests(TestCase):
 
 
 class GetUnstagedChangesTests(TestCase):
-    def test_get_unstaged_changes(self):
+    def test_get_unstaged_changes(self) -> None:
         """Unit test for get_unstaged_changes."""
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
@@ -674,7 +675,7 @@ class GetUnstagedChangesTests(TestCase):
 
             self.assertEqual(list(changes), [b"foo1"])
 
-    def test_get_unstaged_deleted_changes(self):
+    def test_get_unstaged_deleted_changes(self) -> None:
         """Unit test for get_unstaged_changes."""
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
@@ -697,7 +698,7 @@ class GetUnstagedChangesTests(TestCase):
 
             self.assertEqual(list(changes), [b"foo1"])
 
-    def test_get_unstaged_changes_removed_replaced_by_directory(self):
+    def test_get_unstaged_changes_removed_replaced_by_directory(self) -> None:
         """Unit test for get_unstaged_changes."""
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
@@ -722,7 +723,7 @@ class GetUnstagedChangesTests(TestCase):
             self.assertEqual(list(changes), [b"foo1"])
 
     @skipIf(not can_symlink(), "Requires symlink support")
-    def test_get_unstaged_changes_removed_replaced_by_link(self):
+    def test_get_unstaged_changes_removed_replaced_by_link(self) -> None:
         """Unit test for get_unstaged_changes."""
         repo_dir = tempfile.mkdtemp()
         self.addCleanup(shutil.rmtree, repo_dir)
@@ -748,7 +749,7 @@ class GetUnstagedChangesTests(TestCase):
 
 
 class TestValidatePathElement(TestCase):
-    def test_default(self):
+    def test_default(self) -> None:
         self.assertTrue(validate_path_element_default(b"bla"))
         self.assertTrue(validate_path_element_default(b".bla"))
         self.assertFalse(validate_path_element_default(b".git"))
@@ -756,7 +757,7 @@ class TestValidatePathElement(TestCase):
         self.assertFalse(validate_path_element_default(b".."))
         self.assertTrue(validate_path_element_default(b"git~1"))
 
-    def test_ntfs(self):
+    def test_ntfs(self) -> None:
         self.assertTrue(validate_path_element_ntfs(b"bla"))
         self.assertTrue(validate_path_element_ntfs(b".bla"))
         self.assertFalse(validate_path_element_ntfs(b".git"))
@@ -766,7 +767,7 @@ class TestValidatePathElement(TestCase):
 
 
 class TestTreeFSPathConversion(TestCase):
-    def test_tree_to_fs_path(self):
+    def test_tree_to_fs_path(self) -> None:
         tree_path = "délwíçh/foo".encode()
         fs_path = _tree_to_fs_path(b"/prefix/path", tree_path)
         self.assertEqual(
@@ -774,12 +775,12 @@ class TestTreeFSPathConversion(TestCase):
             os.fsencode(os.path.join("/prefix/path", "délwíçh", "foo")),
         )
 
-    def test_fs_to_tree_path_str(self):
+    def test_fs_to_tree_path_str(self) -> None:
         fs_path = os.path.join(os.path.join("délwíçh", "foo"))
         tree_path = _fs_to_tree_path(fs_path)
         self.assertEqual(tree_path, "délwíçh/foo".encode())
 
-    def test_fs_to_tree_path_bytes(self):
+    def test_fs_to_tree_path_bytes(self) -> None:
         fs_path = os.path.join(os.fsencode(os.path.join("délwíçh", "foo")))
         tree_path = _fs_to_tree_path(fs_path)
         self.assertEqual(tree_path, "délwíçh/foo".encode())
