@@ -1,6 +1,7 @@
 # test_web.py -- Tests for the git HTTP server
 # Copyright (C) 2010 Google, Inc.
 #
+# SPDX-License-Identifier: Apache-2.0 OR GPL-2.0-or-later
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
 # General Public License as public by the Free Software Foundation; version 2.0
 # or (at your option) any later version. You can redistribute it and/or
@@ -24,6 +25,7 @@ import gzip
 import os
 import re
 from io import BytesIO
+from typing import NoReturn
 
 from dulwich.object_store import MemoryObjectStore
 from dulwich.objects import Blob
@@ -71,10 +73,10 @@ class MinimalistWSGIInputStream:
 class MinimalistWSGIInputStream2(MinimalistWSGIInputStream):
     """WSGI input stream with no *working* 'seek()' and 'tell()' methods."""
 
-    def seek(self, pos):
+    def seek(self, pos) -> NoReturn:
         raise NotImplementedError
 
-    def tell(self):
+    def tell(self) -> NoReturn:
         raise NotImplementedError
 
 
@@ -85,10 +87,10 @@ class TestHTTPGitRequest(HTTPGitRequest):
         HTTPGitRequest.__init__(self, *args, **kwargs)
         self.cached = None
 
-    def nocache(self):
+    def nocache(self) -> None:
         self.cached = False
 
-    def cache_forever(self):
+    def cache_forever(self) -> None:
         self.cached = True
 
 
@@ -97,7 +99,7 @@ class WebTestCase(TestCase):
 
     _req_class: type[HTTPGitRequest] = TestHTTPGitRequest
 
-    def setUp(self):
+    def setUp(self) -> None:
         super().setUp()
         self._environ = {}
         self._req = self._req_class(
@@ -112,10 +114,10 @@ class WebTestCase(TestCase):
         self._headers = list(headers)
         return self._output.write
 
-    def _handlers(self):
+    def _handlers(self) -> None:
         return None
 
-    def assertContentTypeEquals(self, expected):
+    def assertContentTypeEquals(self, expected) -> None:
         self.assertIn(("Content-Type", expected), self._headers)
 
 
@@ -131,11 +133,11 @@ def _test_backend(objects, refs=None, named_files=None):
 
 
 class DumbHandlersTestCase(WebTestCase):
-    def test_send_file_not_found(self):
+    def test_send_file_not_found(self) -> None:
         list(send_file(self._req, None, "text/plain"))
         self.assertEqual(HTTP_NOT_FOUND, self._status)
 
-    def test_send_file(self):
+    def test_send_file(self) -> None:
         f = BytesIO(b"foobar")
         output = b"".join(send_file(self._req, f, "some/thing"))
         self.assertEqual(b"foobar", output)
@@ -143,7 +145,7 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertContentTypeEquals("some/thing")
         self.assertTrue(f.closed)
 
-    def test_send_file_buffered(self):
+    def test_send_file_buffered(self) -> None:
         bufsize = 10240
         xs = b"x" * bufsize
         f = BytesIO(2 * xs)
@@ -152,16 +154,16 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertContentTypeEquals("some/thing")
         self.assertTrue(f.closed)
 
-    def test_send_file_error(self):
+    def test_send_file_error(self) -> None:
         class TestFile:
             def __init__(self, exc_class) -> None:
                 self.closed = False
                 self._exc_class = exc_class
 
-            def read(self, size=-1):
+            def read(self, size=-1) -> NoReturn:
                 raise self._exc_class
 
-            def close(self):
+            def close(self) -> None:
                 self.closed = True
 
         f = TestFile(IOError)
@@ -176,7 +178,7 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertTrue(f.closed)
         self.assertFalse(self._req.cached)
 
-    def test_get_text_file(self):
+    def test_get_text_file(self) -> None:
         backend = _test_backend([], named_files={"description": b"foo"})
         mat = re.search(".*", "description")
         output = b"".join(get_text_file(self._req, backend, mat))
@@ -185,7 +187,7 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertContentTypeEquals("text/plain")
         self.assertFalse(self._req.cached)
 
-    def test_get_loose_object(self):
+    def test_get_loose_object(self) -> None:
         blob = make_object(Blob, data=b"foo")
         backend = _test_backend([blob])
         mat = re.search("^(..)(.{38})$", blob.id.decode("ascii"))
@@ -195,17 +197,17 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertContentTypeEquals("application/x-git-loose-object")
         self.assertTrue(self._req.cached)
 
-    def test_get_loose_object_missing(self):
+    def test_get_loose_object_missing(self) -> None:
         mat = re.search("^(..)(.{38})$", "1" * 40)
         list(get_loose_object(self._req, _test_backend([]), mat))
         self.assertEqual(HTTP_NOT_FOUND, self._status)
 
-    def test_get_loose_object_error(self):
+    def test_get_loose_object_error(self) -> None:
         blob = make_object(Blob, data=b"foo")
         backend = _test_backend([blob])
         mat = re.search("^(..)(.{38})$", blob.id.decode("ascii"))
 
-        def as_legacy_object_error(self):
+        def as_legacy_object_error(self) -> NoReturn:
             raise OSError
 
         self.addCleanup(setattr, Blob, "as_legacy_object", Blob.as_legacy_object)
@@ -213,7 +215,7 @@ class DumbHandlersTestCase(WebTestCase):
         list(get_loose_object(self._req, backend, mat))
         self.assertEqual(HTTP_ERROR, self._status)
 
-    def test_get_pack_file(self):
+    def test_get_pack_file(self) -> None:
         pack_name = os.path.join("objects", "pack", "pack-%s.pack" % ("1" * 40))
         backend = _test_backend([], named_files={pack_name: b"pack contents"})
         mat = re.search(".*", pack_name)
@@ -223,7 +225,7 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertContentTypeEquals("application/x-git-packed-objects")
         self.assertTrue(self._req.cached)
 
-    def test_get_idx_file(self):
+    def test_get_idx_file(self) -> None:
         idx_name = os.path.join("objects", "pack", "pack-%s.idx" % ("1" * 40))
         backend = _test_backend([], named_files={idx_name: b"idx contents"})
         mat = re.search(".*", idx_name)
@@ -233,7 +235,7 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertContentTypeEquals("application/x-git-packed-objects-toc")
         self.assertTrue(self._req.cached)
 
-    def test_get_info_refs(self):
+    def test_get_info_refs(self) -> None:
         self._environ["QUERY_STRING"] = ""
 
         blob1 = make_object(Blob, data=b"1")
@@ -265,7 +267,7 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertContentTypeEquals("text/plain")
         self.assertFalse(self._req.cached)
 
-    def test_get_info_refs_not_found(self):
+    def test_get_info_refs_not_found(self) -> None:
         self._environ["QUERY_STRING"] = ""
 
         objects = []
@@ -280,7 +282,7 @@ class DumbHandlersTestCase(WebTestCase):
         self.assertEqual(HTTP_NOT_FOUND, self._status)
         self.assertContentTypeEquals("text/plain")
 
-    def test_get_info_packs(self):
+    def test_get_info_packs(self) -> None:
         class TestPackData:
             def __init__(self, sha) -> None:
                 self.filename = f"pack-{sha}.pack"
@@ -326,7 +328,7 @@ class SmartHandlersTestCase(WebTestCase):
             self.stateless_rpc = stateless_rpc
             self.advertise_refs = advertise_refs
 
-        def handle(self):
+        def handle(self) -> None:
             self.proto.write(b"handled input: " + self.proto.recv(1024))
 
     def _make_handler(self, *args, **kwargs):
@@ -336,21 +338,21 @@ class SmartHandlersTestCase(WebTestCase):
     def _handlers(self):
         return {b"git-upload-pack": self._make_handler}
 
-    def test_handle_service_request_unknown(self):
+    def test_handle_service_request_unknown(self) -> None:
         mat = re.search(".*", "/git-evil-handler")
         content = list(handle_service_request(self._req, "backend", mat))
         self.assertEqual(HTTP_FORBIDDEN, self._status)
         self.assertNotIn(b"git-evil-handler", b"".join(content))
         self.assertFalse(self._req.cached)
 
-    def _run_handle_service_request(self, content_length=None):
+    def _run_handle_service_request(self, content_length=None) -> None:
         self._environ["wsgi.input"] = BytesIO(b"foo")
         if content_length is not None:
             self._environ["CONTENT_LENGTH"] = content_length
         mat = re.search(".*", "/git-upload-pack")
 
         class Backend:
-            def open_repository(self, path):
+            def open_repository(self, path) -> None:
                 return None
 
         handler_output = b"".join(handle_service_request(self._req, Backend(), mat))
@@ -363,20 +365,20 @@ class SmartHandlersTestCase(WebTestCase):
         self.assertTrue(self._handler.stateless_rpc)
         self.assertFalse(self._req.cached)
 
-    def test_handle_service_request(self):
+    def test_handle_service_request(self) -> None:
         self._run_handle_service_request()
 
-    def test_handle_service_request_with_length(self):
+    def test_handle_service_request_with_length(self) -> None:
         self._run_handle_service_request(content_length="3")
 
-    def test_handle_service_request_empty_length(self):
+    def test_handle_service_request_empty_length(self) -> None:
         self._run_handle_service_request(content_length="")
 
-    def test_get_info_refs_unknown(self):
+    def test_get_info_refs_unknown(self) -> None:
         self._environ["QUERY_STRING"] = "service=git-evil-handler"
 
         class Backend:
-            def open_repository(self, url):
+            def open_repository(self, url) -> None:
                 return None
 
         mat = re.search(".*", "/git-evil-pack")
@@ -385,12 +387,12 @@ class SmartHandlersTestCase(WebTestCase):
         self.assertEqual(HTTP_FORBIDDEN, self._status)
         self.assertFalse(self._req.cached)
 
-    def test_get_info_refs(self):
+    def test_get_info_refs(self) -> None:
         self._environ["wsgi.input"] = BytesIO(b"foo")
         self._environ["QUERY_STRING"] = "service=git-upload-pack"
 
         class Backend:
-            def open_repository(self, url):
+            def open_repository(self, url) -> None:
                 return None
 
         mat = re.search(".*", "/git-upload-pack")
@@ -413,16 +415,16 @@ class SmartHandlersTestCase(WebTestCase):
 
 
 class LengthLimitedFileTestCase(TestCase):
-    def test_no_cutoff(self):
+    def test_no_cutoff(self) -> None:
         f = _LengthLimitedFile(BytesIO(b"foobar"), 1024)
         self.assertEqual(b"foobar", f.read())
 
-    def test_cutoff(self):
+    def test_cutoff(self) -> None:
         f = _LengthLimitedFile(BytesIO(b"foobar"), 3)
         self.assertEqual(b"foo", f.read())
         self.assertEqual(b"", f.read())
 
-    def test_multiple_reads(self):
+    def test_multiple_reads(self) -> None:
         f = _LengthLimitedFile(BytesIO(b"foobar"), 3)
         self.assertEqual(b"fo", f.read(2))
         self.assertEqual(b"o", f.read(2))
@@ -433,26 +435,26 @@ class HTTPGitRequestTestCase(WebTestCase):
     # This class tests the contents of the actual cache headers
     _req_class = HTTPGitRequest
 
-    def test_not_found(self):
+    def test_not_found(self) -> None:
         self._req.cache_forever()  # cache headers should be discarded
         message = "Something not found"
         self.assertEqual(message.encode("ascii"), self._req.not_found(message))
         self.assertEqual(HTTP_NOT_FOUND, self._status)
         self.assertEqual({("Content-Type", "text/plain")}, set(self._headers))
 
-    def test_forbidden(self):
+    def test_forbidden(self) -> None:
         self._req.cache_forever()  # cache headers should be discarded
         message = "Something not found"
         self.assertEqual(message.encode("ascii"), self._req.forbidden(message))
         self.assertEqual(HTTP_FORBIDDEN, self._status)
         self.assertEqual({("Content-Type", "text/plain")}, set(self._headers))
 
-    def test_respond_ok(self):
+    def test_respond_ok(self) -> None:
         self._req.respond()
         self.assertEqual([], self._headers)
         self.assertEqual(HTTP_OK, self._status)
 
-    def test_respond(self):
+    def test_respond(self) -> None:
         self._req.nocache()
         self._req.respond(
             status=402,
@@ -474,7 +476,7 @@ class HTTPGitRequestTestCase(WebTestCase):
 
 
 class HTTPGitApplicationTestCase(TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         super().setUp()
         self._app = HTTPGitApplication("backend")
 
@@ -483,25 +485,25 @@ class HTTPGitApplicationTestCase(TestCase):
             "REQUEST_METHOD": "GET",
         }
 
-    def _test_handler(self, req, backend, mat):
+    def _test_handler(self, req, backend, mat) -> str:
         # tests interface used by all handlers
         self.assertEqual(self._environ, req.environ)
         self.assertEqual("backend", backend)
         self.assertEqual("/foo", mat.group(0))
         return "output"
 
-    def _add_handler(self, app):
+    def _add_handler(self, app) -> None:
         req = self._environ["REQUEST_METHOD"]
         app.services = {
             (req, re.compile("/foo$")): self._test_handler,
         }
 
-    def test_call(self):
+    def test_call(self) -> None:
         self._add_handler(self._app)
         self.assertEqual("output", self._app(self._environ, None))
 
-    def test_fallback_app(self):
-        def test_app(environ, start_response):
+    def test_fallback_app(self) -> None:
+        def test_app(environ, start_response) -> str:
             return "output"
 
         app = HTTPGitApplication("backend", fallback_app=test_app)
@@ -514,7 +516,7 @@ class GunzipTestCase(HTTPGitApplicationTestCase):
     """
     example_text = __doc__.encode("ascii")
 
-    def setUp(self):
+    def setUp(self) -> None:
         super().setUp()
         self._app = GunzipFilter(self._app)
         self._environ["HTTP_CONTENT_ENCODING"] = "gzip"
@@ -529,7 +531,7 @@ class GunzipTestCase(HTTPGitApplicationTestCase):
         zstream.seek(0)
         return zstream, zlength
 
-    def _test_call(self, orig, zstream, zlength):
+    def _test_call(self, orig, zstream, zlength) -> None:
         self._add_handler(self._app.app)
         self.assertLess(zlength, len(orig))
         self.assertEqual(self._environ["HTTP_CONTENT_ENCODING"], "gzip")
@@ -543,10 +545,10 @@ class GunzipTestCase(HTTPGitApplicationTestCase):
         self.assertIs(None, self._environ.get("CONTENT_LENGTH"))
         self.assertNotIn("HTTP_CONTENT_ENCODING", self._environ)
 
-    def test_call(self):
+    def test_call(self) -> None:
         self._test_call(self.example_text, *self._get_zstream(self.example_text))
 
-    def test_call_no_seek(self):
+    def test_call_no_seek(self) -> None:
         """This ensures that the gunzipping code doesn't require any methods on
         'wsgi.input' except for '.read()'.  (In particular, it shouldn't
         require '.seek()'. See https://github.com/jelmer/dulwich/issues/140.).
@@ -558,7 +560,7 @@ class GunzipTestCase(HTTPGitApplicationTestCase):
             zlength,
         )
 
-    def test_call_no_working_seek(self):
+    def test_call_no_working_seek(self) -> None:
         """Similar to 'test_call_no_seek', but this time the methods are available
         (but defunct).  See https://github.com/jonashaag/klaus/issues/154.
         """
